/*
 * Copyright 2008 Chris Povirk
 * 
 * This file is part of General Manager for Tecmo Super Bowl.
 * 
 * General Manager for Tecmo Super Bowl is free software: you can redistribute
 * it and/or modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 * 
 * General Manager for Tecmo Super Bowl is distributed in the hope that it will
 * be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * General Manager for Tecmo Super Bowl. If not, see
 * <http://www.gnu.org/licenses/>.
 */

package org.twofoos.gmtsb.core.play;

import java.util.Collections;
import java.util.Iterator;
import java.util.SortedMap;
import java.util.TreeMap;

public class Playbook implements Iterable<Play>
{
  private final SortedMap<Page, Play> plays;

  public Playbook()
  {
    this.plays = new TreeMap<Page, Play>();

    for(Page page : Page.values())
    {
      plays.put(page, null);
    }
  }

  public Iterator<Play> iterator()
  {
    return Collections.unmodifiableSortedMap(plays).values().iterator();
  }

  public Play put(Page page, Play play)
  {
    return plays.put(page, play);
  }

  public Play get(Page page)
  {
    return plays.get(page);
  }

  @Override
  public int hashCode()
  {
    return plays.hashCode();
  }

  @Override
  public boolean equals(Object obj)
  {
    if(!(obj instanceof Playbook))
    {
      return false;
    }
    Playbook that = (Playbook) obj;
    return this.plays.equals(that.plays);
  }
}
