/*
 * Copyright 2008 Chris Povirk
 * 
 * This file is part of General Manager for Tecmo Super Bowl.
 * 
 * General Manager for Tecmo Super Bowl is free software: you can redistribute
 * it and/or modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 * 
 * General Manager for Tecmo Super Bowl is distributed in the hope that it will
 * be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * General Manager for Tecmo Super Bowl. If not, see
 * <http://www.gnu.org/licenses/>.
 */

package org.twofoos.gmtsb.gui.util;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.util.HashMap;
import java.util.Map;

import javax.swing.Icon;

public class SolidColorIcon implements Icon
{
  private static final int COLOR_WIDTH = 23;
  private static final int COLOR_HEIGHT = 21;

  private final Color color;

  /**
   * Use getIconForColor().
   */
  private SolidColorIcon(Color color)
  {
    this.color = color;
  }

  private static final Map<Color, Icon> icons = new HashMap<Color, Icon>();

  public static Icon getIconForColor(Color color)
  {
    if(!icons.containsKey(color))
    {
      icons.put(color, new SolidColorIcon(color));
    }

    return icons.get(color);
  }

  public void paintIcon(Component c, Graphics g, int x, int y)
  {
    g.setColor(color);
    g.fillRect(x, y, COLOR_WIDTH, COLOR_HEIGHT);
  }

  public int getIconWidth()
  {
    return COLOR_WIDTH;
  }

  public int getIconHeight()
  {
    return COLOR_HEIGHT;
  }
}
